from __future__ import annotations
from threading import Lock, Thread
from typing import Optional
# -*- coding: utf-8 -*-
"""
Created on Mon Feb  7 19:50:47 2022

@author: krein_000
"""
from intern.gng_klassen import Bild, Text, Ereignisbehandlung
import time


class SingletonMeta(type):
    """ 
    Hilfsklasse zur Realisierung des Singleton-Musters, damit zu jeder Zeit nur ein einziges Objekt der Klasse Zeichenfenster existiert. 
    """
    _instanz: Optional[Singleton] = None
    _lock: Lock = Lock()


    def __call__(cls, *args, **kwargs):
        """ 
        Methode sorgt dafür, dass nur ein Objekt der Klasse ausgegeben wird.
        """
        with cls._lock:
            if not cls._instanz:
                cls._instanz = super().__call__(*args, **kwargs)
        return cls._instanz



class MethodenboxIntern(Ereignisbehandlung, Thread, metaclass=SingletonMeta):


    def __init__(self):
        """
        Erzeugt eine neue Methodenbox für das Arbeiten mit Bildern und Textfeldern
        """
        super().__init__()
        self.textliste = []
        super().Starten()



    def fuegeTextHinzu(self, name, text, xPos, yPos):
        """
        Hinzufügen eines Textfeldes
        --Parameter name eindeutiger Bezeichner des Textfelds
        --Parameter text darzustellender Text
        --Parameter xPos x-Position der linken oberen Ecke des Textfeldes 
        --Parameter yPos y-Position der linken oberen Ecke des Textfeldes
       """    
        for t in self.textliste:
            if t[0] == name:
                #print("Eindeutige Textname bereits vergeben. Text wird nicht eingefügt")
                return 
        textneu = Text(xPos, yPos, 0, 12, "schwarz", True)
        textneu.TextSetzen(text)
        textneu.TextGroesseSetzen(30)
        self.textliste.append([name, textneu])
        
    def fuegeTextHinzu(self, name, text, xPos, yPos, farbe):
        """
        Hinzufügen eines Textfeldes
        --Parameter name eindeutiger Bezeichner des Textfelds
        --Parameter text darzustellender Text
        --Parameter xPos x-Position der linken oberen Ecke des Textfeldes 
        --Parameter yPos y-Position der linken oberen Ecke des Textfeldes
        --Parameter farbe Farbe der Schrift 
        """    
        for t in self.textliste:
            if t[0] == name:
                #print("Eindeutige Textname bereits vergeben. Text wird nicht eingefügt")
                return 
        textneu = Text(xPos, yPos, 0, 12, "schwarz", True)
        textneu.TextSetzen(text)
        textneu.TextGroesseSetzen(30)
        textneu.FarbeSetzen(farbe)
        self.textliste.append([name, textneu])
        
    


    def entferneText(self, name):
        """
        Methode zum Entfernen des Textfeldes
        --Parameter name der eindeutige Bezeichner des Textfelds
        """  
        for t in self.textliste:
            if t[0] == name:
                t[1].SichtbarkeitSetzen(False)
                t[1].Entfernen()
                self.textliste.remove(t)
                return
            

    def ersetzeText(self, name, textNeu):
        """
        Methode zum Ersetzen des Textes im Textfeld
        --Parameter name der eindeutige Bezeichner des Textfelds
        --Parameter textNeu der neue Text
        """    
        for t in self.textliste:
            if t[0] == name:
                t[1].TextSetzen(textNeu)
            

    def setzeTextposition(self, name, xPos, yPos):
        """
        Methode zum Ändern der Position des Textfeldes
        --Parameter name der eindeutige Bezeichner des Textfelds
        --Parameter xPos neue x-Position
        --Parameter yPos neue y-Position
        """   
        for t in self.textliste:
            if t[0] == name:     
                t[1].PositionSetzen(xPos, yPos)
                
    def setzeSichtbarkeitText(self, name, sichtbarkeitNeu):
        """
        Methode zum Ändern der Position des Textfeldes
        --Parameter name der eindeutige Bezeichner des Textfelds
        --Parameter sichtbarkeitNeu neue Sichtbarkeit
        """   
        for t in self.textliste:
            if t[0] == name:     
                t[1].SichtbarkeitSetzen(sichtbarkeitNeu)
            
   
    def verschiebe(self, name, deltaX, deltaY):
        """
        verschiebt den Text
        --Parameter name der eindeutige Bezeichner des Textfelds   
        --Parameter deltaX Verschiebung in x-Richtung
        --Parameter deltaY Verschiebung in y-Richtung
        """
        for t in self.textliste:
            if t[0] == name:     
                t[1].Verschieben(deltaX, deltaY)
        
    def drehe(self, name, winkel):
         """
         Dreht den Text
         --Parameter winkel Drehwinkel
         """
         for t in self.textliste:
            if t[0] == name:     
                t[1].Drehen(winkel)  
            